"""A lightweight pinch analysis package"""

from pina.pinch_analyzer import PinchAnalyzer
from pina.stream import make_segmented_stream, make_stream

__version__ = "0.1.0"

__all__ = ["PinchAnalyzer", "make_segmented_stream", "make_stream"]
