#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pina', 'pina.segments']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

extras_require = \
{'dev': ['black', 'coverage', 'flake8', 'isort', 'pre-commit', 'tox']}

setup(name='pina',
      version='0.1.0',
      description='A lightweight pinch analysis package',
      author='Patrick Heck',
      author_email='fitz_oreilly@outlook.com',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
